import copy
from dataclasses import dataclass
from typing import Any, Dict, Iterable, Mapping


@dataclass
class RemoteComponentType:
    name: str

    @property
    def key(self) -> str:
        return self.name
        # return f"{self.package}.{self.name}"


class RemoteComponentRegistry:
    @classmethod
    def from_dict(cls, components: Dict[str, Mapping[str, Any]]) -> "RemoteComponentRegistry":
        return RemoteComponentRegistry(
            {key: RemoteComponentType(**value) for key, value in components.items()}
        )

    def __init__(self, components: Dict[str, RemoteComponentType]):
        self._components: Dict[str, RemoteComponentType] = copy.copy(components)

    @staticmethod
    def empty() -> "RemoteComponentRegistry":
        return RemoteComponentRegistry({})

    def has(self, name: str) -> bool:
        return name in self._components

    def get(self, name: str) -> RemoteComponentType:
        return self._components[name]

    def keys(self) -> Iterable[str]:
        return self._components.keys()

    def __repr__(self) -> str:
        return f"<RemoteComponentRegistry {list(self._components.keys())}>"



import os
import sys
from pathlib import Path
from typing import Optional, Tuple

import click

from dg_cli.context import (
    CodeLocationProjectContext,
    DeploymentProjectContext,
    is_inside_code_location_project,
    is_inside_deployment_project,
)
from dg_cli.generate import (
    generate_code_location,
    generate_component_instance,
    generate_component_type,
    generate_deployment,
)


@click.group(name="generate")
def generate_cli() -> None:
    """Commands for generating Dagster components and related entities."""


@generate_cli.command(name="deployment")
@click.argument("path", type=str)
def generate_deployment_command(path: str) -> None:
    """Generate a Dagster deployment instance."""
    dir_abspath = os.path.abspath(path)
    if os.path.exists(dir_abspath):
        click.echo(
            click.style(f"A file or directory at {dir_abspath} already exists. ", fg="red")
            + "\nPlease delete the contents of this path or choose another location."
        )
        sys.exit(1)
    generate_deployment(path)


@generate_cli.command(name="code-location")
@click.argument("name", type=str)
@click.option("--use-editable-dagster", is_flag=True, default=False)
def generate_code_location_command(name: str, use_editable_dagster: bool) -> None:
    """Generate a Dagster code location inside a component."""
    if is_inside_deployment_project(Path.cwd()):
        context = DeploymentProjectContext.from_path(Path.cwd())
        if context.has_code_location(name):
            click.echo(click.style(f"A code location named {name} already exists.", fg="red"))
            sys.exit(1)
        code_location_path = os.path.join(context.code_location_root_path, name)
    else:
        code_location_path = os.path.join(Path.cwd(), name)

    if use_editable_dagster:
        if "DAGSTER_GIT_REPO_DIR" not in os.environ:
            click.echo(
                click.style(
                    "The `--use-editable-dagster` flag requires the `DAGSTER_GIT_REPO_DIR` environment variable to be set.",
                    fg="red",
                )
            )
            sys.exit(1)
        editable_dagster_root = os.environ["DAGSTER_GIT_REPO_DIR"]
    else:
        editable_dagster_root = None

    generate_code_location(code_location_path, editable_dagster_root)


@generate_cli.command(name="component-type")
@click.argument("name", type=str)
def generate_component_type_command(name: str) -> None:
    """Generate a Dagster component instance."""
    if not is_inside_code_location_project(Path.cwd()):
        click.echo(
            click.style(
                "This command must be run inside a Dagster code location project.", fg="red"
            )
        )
        sys.exit(1)
    context = CodeLocationProjectContext.from_path(Path.cwd())
    full_component_name = f"{context.name}.{name}"
    if context.has_component_type(full_component_name):
        click.echo(click.style(f"A component type named `{name}` already exists.", fg="red"))
        sys.exit(1)

    generate_component_type(context.component_types_root_path, name)


@generate_cli.command(name="component")
@click.argument("component_type", type=str)
@click.argument("component_name", type=str)
@click.option("--json-params", type=str, default=None)
@click.argument("extra_args", nargs=-1, type=str)
def generate_component_command(
    component_type: str,
    component_name: str,
    json_params: Optional[str],
    extra_args: Tuple[str, ...],
) -> None:
    if not is_inside_code_location_project(Path.cwd()):
        click.echo(
            click.style(
                "This command must be run inside a Dagster code location project.", fg="red"
            )
        )
        sys.exit(1)

    context = CodeLocationProjectContext.from_path(Path.cwd())
    if not context.has_component_type(component_type):
        click.echo(
            click.style(f"No component type `{component_type}` could be resolved.", fg="red")
        )
        sys.exit(1)
    elif context.has_component_instance(component_name):
        click.echo(
            click.style(f"A component instance named `{component_name}` already exists.", fg="red")
        )
        sys.exit(1)

    generate_component_instance(
        context.component_instances_root_path,
        component_name,
        component_type,
        json_params,
        extra_args,
    )



from pathlib import Path

from dagster_components.core.component_defs_builder import (
    build_defs_from_toplevel_components_folder,
)

defs = build_defs_from_toplevel_components_folder(
    path=Path(__file__).parent,
)



import sys
from pathlib import Path

import click

from dg_cli.context import (
    CodeLocationProjectContext,
    DeploymentProjectContext,
    is_inside_code_location_project,
    is_inside_deployment_project,
)


@click.group(name="list")
def list_cli():
    """Commands for listing Dagster components and related entities."""


@list_cli.command(name="code-locations")
def list_code_locations_command() -> None:
    """List code locations in the current deployment."""
    if not is_inside_deployment_project(Path.cwd()):
        click.echo(
            click.style("This command must be run inside a Dagster deployment project.", fg="red")
        )
        sys.exit(1)

    context = DeploymentProjectContext.from_path(Path.cwd())
    for code_location in context.list_code_locations():
        click.echo(code_location)


@list_cli.command(name="component-types")
def list_component_types_command() -> None:
    """List registered Dagster components."""
    if not is_inside_code_location_project(Path.cwd()):
        click.echo(
            click.style(
                "This command must be run inside a Dagster code location project.", fg="red"
            )
        )
        sys.exit(1)

    context = CodeLocationProjectContext.from_path(Path.cwd())
    for component_type in context.list_component_types():
        click.echo(component_type)


@list_cli.command(name="components")
def list_components_command() -> None:
    """List Dagster component instances in a code location."""
    if not is_inside_code_location_project(Path.cwd()):
        click.echo(
            click.style(
                "This command must be run inside a Dagster code location project.", fg="red"
            )
        )
        sys.exit(1)

    context = CodeLocationProjectContext.from_path(Path.cwd())
    for component_name in context.component_instances:
        click.echo(component_name)



import json
import os
from pathlib import Path
from typing import Final, Iterable, Mapping, Optional, Sequence

import tomli
from typing_extensions import Self

from dg_cli.component import RemoteComponentRegistry, RemoteComponentType
from dg_cli.error import DgError
from dg_cli.utils import execute_code_location_command


def is_inside_deployment_project(path: Path) -> bool:
    try:
        _resolve_deployment_root_path(path)
        return True
    except DgError:
        return False


def _resolve_deployment_root_path(path: Path) -> Path:
    current_path = path.absolute()
    while not _is_deployment_root(current_path):
        current_path = current_path.parent
        if str(current_path) == "/":
            raise DgError("Cannot find deployment root")
    return current_path


def is_inside_code_location_project(path: Path) -> bool:
    try:
        _resolve_code_location_root_path(path)
        return True
    except DgError:
        return False


def _resolve_code_location_root_path(path: Path) -> Path:
    current_path = path.absolute()
    while not _is_code_location_root(current_path):
        current_path = current_path.parent
        if str(current_path) == "/":
            raise DgError("Cannot find code location root")
    return current_path


def _is_deployment_root(path: Path) -> bool:
    return (path / "code_locations").exists()


def _is_code_location_root(path: Path) -> bool:
    if (path / "pyproject.toml").exists():
        with open(path / "pyproject.toml") as f:
            toml = tomli.loads(f.read())
            return bool(toml.get("tool", {}).get("dagster"))
    return False


# Deployment
_DEPLOYMENT_CODE_LOCATIONS_DIR: Final = "code_locations"

# Code location
_CODE_LOCATION_CUSTOM_COMPONENTS_DIR: Final = "lib"
_CODE_LOCATION_COMPONENT_INSTANCES_DIR: Final = "components"


class DeploymentProjectContext:
    @classmethod
    def from_path(cls, path: Path) -> Self:
        return cls(root_path=_resolve_deployment_root_path(path))

    def __init__(self, root_path: Path):
        self._root_path = root_path

    @property
    def deployment_root(self) -> Path:
        return self._root_path

    @property
    def code_location_root_path(self) -> Path:
        return self._root_path / _DEPLOYMENT_CODE_LOCATIONS_DIR

    def has_code_location(self, name: str) -> bool:
        return os.path.exists(os.path.join(self._root_path, "code_locations", name))

    def list_code_locations(self) -> Iterable[str]:
        return sorted(os.listdir(os.path.join(self._root_path, "code_locations")))


class CodeLocationProjectContext:
    _components_registry: Mapping[str, RemoteComponentType] = {}

    @classmethod
    def from_path(cls, path: Path) -> Self:
        root_path = _resolve_code_location_root_path(path)
        raw_component_registry = execute_code_location_command(
            root_path, ["list", "component-types"]
        )
        component_registry = RemoteComponentRegistry.from_dict(json.loads(raw_component_registry))
        deployment_context = (
            DeploymentProjectContext.from_path(path) if is_inside_deployment_project(path) else None
        )

        return cls(
            deployment_context=deployment_context,
            root_path=root_path,
            name=path.name,
            component_registry=component_registry,
        )

    def __init__(
        self,
        deployment_context: Optional[DeploymentProjectContext],
        root_path: Path,
        name: str,
        component_registry: "RemoteComponentRegistry",
    ):
        self._deployment_context = deployment_context
        self._root_path = root_path
        self._name = name
        self._component_registry = component_registry

    @property
    def name(self) -> str:
        return self._name

    @property
    def deployment_context(self) -> Optional[DeploymentProjectContext]:
        return self._deployment_context

    @property
    def component_types_root_path(self) -> str:
        return os.path.join(self._root_path, self._name, _CODE_LOCATION_CUSTOM_COMPONENTS_DIR)

    @property
    def component_types_root_module(self) -> str:
        return f"{self._name}.{_CODE_LOCATION_CUSTOM_COMPONENTS_DIR}"

    @property
    def component_registry(self) -> "RemoteComponentRegistry":
        return self._component_registry

    def has_component_type(self, name: str) -> bool:
        return self._component_registry.has(name)

    def get_component_type(self, name: str) -> RemoteComponentType:
        if not self.has_component_type(name):
            raise DgError(f"No component type named {name}")
        return self._component_registry.get(name)

    def list_component_types(self) -> Sequence[str]:
        return sorted(self._component_registry.keys())

    def get_component_instance_path(self, name: str) -> str:
        if name not in self.component_instances:
            raise DgError(f"No component instance named {name}")
        return os.path.join(self.component_instances_root_path, name)

    @property
    def component_instances_root_path(self) -> str:
        return os.path.join(self._root_path, self._name, _CODE_LOCATION_COMPONENT_INSTANCES_DIR)

    @property
    def component_instances(self) -> Iterable[str]:
        return os.listdir(
            os.path.join(self._root_path, self._name, _CODE_LOCATION_COMPONENT_INSTANCES_DIR)
        )

    def has_component_instance(self, name: str) -> bool:
        return os.path.exists(
            os.path.join(self._root_path, self._name, _CODE_LOCATION_COMPONENT_INSTANCES_DIR, name)
        )



import click

from dg_cli.cli.generate import generate_cli
from dg_cli.cli.list import list_cli
from dg_cli.version import __version__


def create_dg_cli():
    commands = {
        "generate": generate_cli,
        "list": list_cli,
    }

    @click.group(
        commands=commands,
        context_settings={"max_content_width": 120, "help_option_names": ["-h", "--help"]},
    )
    @click.version_option(__version__, "--version", "-v")
    def group():
        """CLI tools for working with Dagster."""

    return group


ENV_PREFIX = "DG_CLI"
cli = create_dg_cli()


def main():
    cli(auto_envvar_prefix=ENV_PREFIX)



import os
import textwrap
from pathlib import Path
from typing import Optional, Tuple

import click

from dg_cli.utils import (
    camelcase,
    discover_git_root,
    execute_code_location_command,
    generate_subtree,
)


def generate_deployment(path: str) -> None:
    click.echo(f"Creating a Dagster deployment at {path}.")

    generate_subtree(
        path=path,
        name_placeholder="DEPLOYMENT_NAME_PLACEHOLDER",
        templates_path=os.path.join(
            os.path.dirname(__file__), "templates", "DEPLOYMENT_NAME_PLACEHOLDER"
        ),
    )


def generate_code_location(path: str, editable_dagster_root: Optional[str] = None) -> None:
    click.echo(f"Creating a Dagster code location at {path}.")

    # Temporarily we always set an editable dagster root. This is needed while the packages are not
    # published.
    editable_dagster_root = (
        editable_dagster_root
        or os.environ.get("DAGSTER_GIT_REPO_DIR")
        or discover_git_root(Path(__file__))
    )

    editable_dagster_uv_sources = textwrap.dedent(f"""
    [tool.uv.sources]
    dagster = {{ path = "{editable_dagster_root}/python_modules/dagster", editable = true }}
    dagster-components = {{ path = "{editable_dagster_root}/python_modules/libraries/dagster-components", editable = true }}
    dagster-pipes = {{ path = "{editable_dagster_root}/python_modules/dagster-pipes", editable = true }}
    dagster-webserver = {{ path = "{editable_dagster_root}/python_modules/dagster-webserver", editable = true }}
    dagster-embedded-elt = {{ path = "{editable_dagster_root}/python_modules/libraries/dagster-embedded-elt", editable = true }}
    dagster-dbt = {{ path = "{editable_dagster_root}/python_modules/libraries/dagster-dbt", editable = true }}
    """)

    if editable_dagster_root:
        uv_sources = editable_dagster_uv_sources
    else:
        uv_sources = editable_dagster_uv_sources

    generate_subtree(
        path=path,
        name_placeholder="CODE_LOCATION_NAME_PLACEHOLDER",
        templates_path=os.path.join(
            os.path.dirname(__file__), "templates", "CODE_LOCATION_NAME_PLACEHOLDER"
        ),
        uv_sources=uv_sources,
    )


def generate_component_type(root_path: str, name: str) -> None:
    click.echo(f"Creating a Dagster component type at {root_path}/{name}.py.")

    generate_subtree(
        path=root_path,
        name_placeholder="COMPONENT_TYPE_NAME_PLACEHOLDER",
        templates_path=os.path.join(os.path.dirname(__file__), "templates", "COMPONENT_TYPE"),
        project_name=name,
        component_type_class_name=camelcase(name),
        component_type=name,
    )


def generate_component_instance(
    root_path: str,
    name: str,
    component_type: str,
    json_params: Optional[str],
    extra_args: Tuple[str, ...],
) -> None:
    click.echo(f"Creating a Dagster component instance at {root_path}/{name}.py.")

    component_instance_root_path = os.path.join(root_path, name)
    generate_subtree(
        path=component_instance_root_path,
        name_placeholder="COMPONENT_INSTANCE_NAME_PLACEHOLDER",
        templates_path=os.path.join(
            os.path.dirname(__file__), "templates", "COMPONENT_INSTANCE_NAME_PLACEHOLDER"
        ),
        project_name=name,
        component_type=component_type,
    )

    code_location_command = (
        "generate",
        "component",
        component_type,
        name,
        *([f"--json-params={json_params}"] if json_params else []),
        *(["--", *extra_args] if extra_args else []),
    )
    execute_code_location_command(Path(component_instance_root_path), code_location_command)



import contextlib
import os
import posixpath
import re
import subprocess
from pathlib import Path
from typing import Any, Final, Iterator, List, Optional, Sequence, Union

import click
import jinja2

from dg_cli.version import __version__ as dagster_version

_CODE_LOCATION_COMMAND_PREFIX: Final = ["uv", "run", "dagster-components"]


def discover_git_root(path: Path) -> str:
    while path != path.parent:
        if (path / ".git").exists():
            return str(path)
        path = path.parent
    raise ValueError("Could not find git root")


def execute_code_location_command(path: Path, cmd: Sequence[str]) -> str:
    with pushd(path):
        full_cmd = [*_CODE_LOCATION_COMMAND_PREFIX, *cmd]
        result = subprocess.run(full_cmd, stdout=subprocess.PIPE, check=False)
        return result.stdout.decode("utf-8")


@contextlib.contextmanager
def pushd(path: Union[str, Path]) -> Iterator[None]:
    old_cwd = os.getcwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(old_cwd)


# Adapted from https://github.com/okunishinishi/python-stringcase/blob/master/stringcase.py
def camelcase(string: str) -> str:
    string = re.sub(r"^[\-_\.]", "", str(string))
    if not string:
        return string
    return str(string[0]).upper() + re.sub(
        r"[\-_\.\s]([a-z])", lambda matched: str(matched.group(1)).upper(), string[1:]
    )


def snakecase(string: str) -> str:
    # Add an underscore before capital letters and lower the case
    string = re.sub(r"(?<!^)(?=[A-Z])", "_", string).lower()
    # Replace any non-alphanumeric characters with underscores
    string = re.sub(r"[^a-z0-9_]", "_", string)
    return string


DEFAULT_EXCLUDES: List[str] = [
    "__pycache__",
    ".pytest_cache",
    "*.egg-info",
    ".DS_Store",
    ".ruff_cache",
    "tox.ini",
    ".gitkeep",  # dummy file that allows empty directories to be checked into git
]

PROJECT_NAME_PLACEHOLDER = "PROJECT_NAME_PLACEHOLDER"


# Copied from dagster._generate.generate
def generate_subtree(
    path: str,
    excludes: Optional[List[str]] = None,
    name_placeholder: str = PROJECT_NAME_PLACEHOLDER,
    templates_path: str = PROJECT_NAME_PLACEHOLDER,
    project_name: Optional[str] = None,
    **other_template_vars: Any,
):
    """Renders templates for Dagster project."""
    excludes = DEFAULT_EXCLUDES if not excludes else DEFAULT_EXCLUDES + excludes

    normalized_path = os.path.normpath(path)
    project_name = project_name or os.path.basename(normalized_path).replace("-", "_")
    if not os.path.exists(normalized_path):
        os.mkdir(normalized_path)

    project_template_path: str = os.path.join(
        os.path.dirname(__file__), "templates", templates_path
    )
    loader: jinja2.loaders.FileSystemLoader = jinja2.FileSystemLoader(
        searchpath=project_template_path
    )
    env: jinja2.environment.Environment = jinja2.Environment(loader=loader)

    # merge custom skip_files with the default list
    for root, dirs, files in os.walk(project_template_path):
        # For each subdirectory in the source template, create a subdirectory in the destination.
        for dirname in dirs:
            src_dir_path: str = os.path.join(root, dirname)
            if _should_skip_file(src_dir_path, excludes):
                continue

            src_relative_dir_path: str = os.path.relpath(src_dir_path, project_template_path)
            dst_relative_dir_path: str = src_relative_dir_path.replace(
                name_placeholder,
                project_name,
                1,
            )
            dst_dir_path: str = os.path.join(normalized_path, dst_relative_dir_path)

            os.mkdir(dst_dir_path)

        # For each file in the source template, render a file in the destination.
        for filename in files:
            src_file_path = os.path.join(root, filename)
            if _should_skip_file(src_file_path, excludes):
                continue

            src_relative_file_path: str = os.path.relpath(src_file_path, project_template_path)
            dst_relative_file_path: str = src_relative_file_path.replace(
                name_placeholder,
                project_name,
                1,
            )
            dst_file_path: str = os.path.join(normalized_path, dst_relative_file_path)

            if dst_file_path.endswith(".jinja"):
                dst_file_path = dst_file_path[: -len(".jinja")]

            with open(dst_file_path, "w", encoding="utf8") as f:
                # Jinja template names must use the POSIX path separator "/".
                template_name: str = src_relative_file_path.replace(os.sep, posixpath.sep)
                template: jinja2.environment.Template = env.get_template(name=template_name)
                f.write(
                    template.render(
                        repo_name=project_name,  # deprecated
                        code_location_name=project_name,
                        dagster_version=dagster_version,
                        project_name=project_name,
                        **other_template_vars,
                    )
                )
                f.write("\n")

    click.echo(f"Generated files for Dagster project in {path}.")


def _should_skip_file(path: str, excludes: List[str] = DEFAULT_EXCLUDES):
    """Given a file path `path` in a source template, returns whether or not the file should be skipped
    when generating destination files.

    Technically, `path` could also be a directory path that should be skipped.
    """
    for pattern in excludes:
        if pattern.lower() in path.lower():
            return True

    return False
